/***************************************************************************
 *   $Id: mscr.cpp,v 1.13 2017-12-13 11:16:52 giacomo Exp $
 *                                                                         *
 *   Copyright (C) 2009 by Fabio Asnicar  fabio.asnicar@elettra.trieste.it *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include "mscr.h"
#include <qtango.h>

mscr::mscr(QWidget *parent) : QWidget(parent) {
    ui.setupUi(this);
    timer = new QTimer(this);

    connect(timer,                    SIGNAL(timeout()), this, SLOT(refresh()));
    connect(ui.position0_radioButton, SIGNAL(clicked()), this, SLOT(setPosition0()));
    connect(ui.position1_radioButton, SIGNAL(clicked()), this, SLOT(setPosition1()));
    connect(ui.position2_radioButton, SIGNAL(clicked()), this, SLOT(setPosition2()));
    connect(ui.position3_radioButton, SIGNAL(clicked()), this, SLOT(setPosition3()));
    connect(ui.position4_radioButton, SIGNAL(clicked()), this, SLOT(setPosition4()));
    connect(ui.lightLevel_comboBox,   SIGNAL(currentIndexChanged(int)), this, SLOT(setLightLevel(int)));
    connect(ui.lightLevel_comboBox_2,   SIGNAL(currentIndexChanged(int)), this, SLOT(setLightLevel2(int)));

    DeviceData dd;
    if (QApplication::arguments().size() > 1) {
        try {
            tangodev = QApplication::arguments()[1];
            server = DeviceFactory::instance()->create(tangodev);
            server->read_attribute("MscrStatus") >> mscrStatus;
            dd = server->command_inout("GetInstrumentList");
            dd >> instruments;
            setPolarAttenuation();
            std::string contr_type;
            server->read_attribute("ControllerType") >> contr_type;
            if (contr_type.substr(0,5) != "IFC-B"){
                //ui.tabWidget->setTabEnabled(3,false);
                ui.tabWidget->removeTab(3);
            }

        } catch (Tango::DevFailed& ex) {
            simpleLog::error("mscr::mscr", ex.errors);
            exit(1);
        } catch (...) {
            simpleLog::error("mscr::mscr", "Unexpected exception occurred.");
            exit(1);
        }
    } else {
        usage();
        exit(1);
    }

    initWidgets();
    refresh();
    timer->start(1000);
}

mscr::~mscr() {
}

void mscr::usage() {
    cout << endl;
    cout << "Usage:" << endl;
    cout << "  mscr device" << endl;
    cout << endl;
}

void mscr::initWidgets() {

    ui.polar2Status_groupBox->setVisible(mscrStatus[3]);//2
    ui.attenuator2_groupBox->setVisible(mscrStatus[3]);//2
    ui.lightLevel_comboBox_2->setVisible(mscrStatus[3]);//2
    ui.tLabelLightLevel_2->setVisible(mscrStatus[3]);//2
    ui.label_41->setVisible(mscrStatus[3]);//2

    ui.stepperVerStatus_groupBox->setVisible(mscrStatus[5]);//4
    ui.verSlit_groupBox->setVisible(mscrStatus[5]);//4
    ui.verSlit_groupBox->setEnabled(mscrStatus[6]);//5
    ui.stepperHorStatus_groupBox->setVisible(mscrStatus[5]);//4
    ui.horSlit_groupBox->setVisible(mscrStatus[5]);//4
    ui.horSlit_groupBox->setEnabled(mscrStatus[7]);//4

    ui.position0_radioButton->setText(QString(instruments[0].c_str()));
    ui.position1_radioButton->setText(QString(instruments[1].c_str()));
    if (instruments.size() > 2) {
        ui.position2_radioButton->setText(QString(instruments[2].c_str()));
    } else {
        ui.position2_radioButton->setHidden(true);
    }
    if (instruments.size() > 3) {
        ui.position3_radioButton->setText(QString(instruments[3].c_str()));
    } else {
        ui.position3_radioButton->setHidden(true);
    }
    if (instruments.size() > 4) {
        ui.position4_radioButton->setText(QString(instruments[4].c_str()));
    } else {
        ui.position4_radioButton->setHidden(true);
    }
}

void mscr::setPolarAttenuation() {
    //  #1
    if (mscrStatus[1]) {  // discrete
        ui.polar1Attenuation_tReaderWriter->setWriterType(TReaderWriter::ComboBox);
    }
    ui.polar1Attenuation_tReaderWriter->setSource("$1/Polar1Attenuation");

    //  #2
    if (mscrStatus[2] and mscrStatus[3]) { // present & discrete
        ui.polar2Attenuation_tReaderWriter->setWriterType(TReaderWriter::ComboBox);
    }
    ui.polar2Attenuation_tReaderWriter->setSource("$1/Polar2Attenuation");
}

void mscr::refresh() {
    try {
        // position -------------
        server->read_attribute("Selector") >> position;
        QFont fontNorma, fontBold;
        fontNorma.setBold(false);
        fontBold.setBold(true);
        qDebug() << "mscr::refresh  position=" << position;

        ui.position0_radioButton->setFont(fontNorma);
        ui.position1_radioButton->setFont(fontNorma);
        ui.position2_radioButton->setFont(fontNorma);
        ui.position3_radioButton->setFont(fontNorma);
        ui.position4_radioButton->setFont(fontNorma);

        switch (position) {
        case 0:
            ui.position0_radioButton->setChecked(true);
            ui.position0_radioButton->setFont(fontBold);
            break;
        case 1:
            ui.position1_radioButton->setChecked(true);
            ui.position1_radioButton->setFont(fontBold);
            break;
        case 2:
            ui.position2_radioButton->setChecked(true);
            ui.position2_radioButton->setFont(fontBold);
            break;
        case 3:
            ui.position3_radioButton->setChecked(true);
            ui.position3_radioButton->setFont(fontBold);
            break;
        case 4:
            ui.position4_radioButton->setChecked(true);
            ui.position4_radioButton->setFont(fontBold);
        }
        // light    -------------
        short lightLevel;
        server->read_attribute("LightLevel") >> lightLevel;
        ui.lightLevel_comboBox->setCurrentIndex((int)lightLevel);
        
        // light 2   -------------
        if (ui.lightLevel_comboBox_2->isVisible()){
            short lightLevel2;
            server->read_attribute("LightLevel2") >> lightLevel2;
            ui.lightLevel_comboBox_2->setCurrentIndex((int)lightLevel2);
        }
        // slits    -------------
        vector<bool> mscrStatus;
        server->read_attribute("MscrStatus") >> mscrStatus;
        ui.verSlit_groupBox->setEnabled(mscrStatus[5]);
        ui.horSlit_groupBox->setEnabled(mscrStatus[6]);
    } catch (Tango::DevFailed& ex) {
        simpleLog::error("mscr::refresh", ex.errors);
    } catch (...) {
        simpleLog::error("mscr::refresh", "Unexpected exception occurred on reading the attribute.");
    }
}


void mscr::setPosition0() {
    if (position != 0) {
        setPosition(0);
    }
}
void mscr::setPosition1() {
    if (position != 1) {
        setPosition(1);
    }
}
void mscr::setPosition2() {
    if (position != 2) {
        setPosition(2);
    }
}
void mscr::setPosition3() {
    if (position != 3) {
        setPosition(3);
    }
}
void mscr::setPosition4() {
    if (position != 4) {
        setPosition(4);
    }
}
void mscr::setPosition(short newPosition) {
    try {
        Tango::DeviceAttribute attr("Selector", newPosition);
        server->write_attribute(attr);
    } catch (...) {
        simpleLog::error("mscr::setPosition", "Unexpected exception occurred on writing the attribute 'Selector'.");
    }
}

void mscr::setLightLevel(int newLevel) {
    try {
        Tango::DeviceAttribute attr("LightLevel", (short int)newLevel);
        server->write_attribute(attr);
    } catch (...) {
        simpleLog::error("mscr::setLightLevel", "Unexpected exception occurred on writing the attribute 'LightLevel'.");
    }
}

void mscr::setLightLevel2(int newLevel) {
    try {
        Tango::DeviceAttribute attr("LightLevel2", (short int)newLevel);
        server->write_attribute(attr);
    } catch (...) {
        simpleLog::error("mscr::setLightLevel2", "Unexpected exception occurred on writing the attribute 'LightLevel2'.");
    }
}

